@@ -8,7 +8,7 @@ no warnings 'uninitialized';
 use Date::Calc qw(Days_in_Month Decode_Day_of_Week Nth_Weekday_of_Month_Year);
 use DateTime;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 sub new {
     my $class = shift;
@@ -28,27 +28,19 @@ sub parse_datetime {
         ($date_string) = @_;
     }
 
-    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday) = localtime;
-    $month++; $year += 1900;
-
-    $self->{sec}   = $sec;
-    $self->{min}   = $min;
-    $self->{hour}  = $hour;
-    $self->{day}   = $day;
-    $self->{month} = $month;
-    $self->{year}  = $year;
-    $self->{wday}  = $wday;
-    $self->{yday}  = $yday;
+    $self->{datetime} = DateTime->now(time_zone => 'local');
 
     $date_string =~ tr/,//d;
 
+    $self->{date_string} = $date_string;
+
     if ($date_string =~ m!/!) {
         my @bits = split '\/', $date_string;
 
         if (scalar @bits == 3) {
-            $self->{day}   = $bits[0];
-            $self->{month} = $bits[1];
-            $self->{year}  = $bits[2];
+            $self->{datetime}->set_day($bits[0]);
+            $self->{datetime}->set_month($bits[1]);
+            $self->{datetime}->set_year($bits[2]);
 
             return $self->_return_dt_object;
         }
@@ -75,14 +67,14 @@ sub parse_datetime {
         }
 
         if ($self->{tokens}->[$self->{index}+1] =~ /^(\d{4})$/) {
-            $self->{year}  = $1;
+            $self->{datetime}->set_year($1);
         }
 
-        $self->_months();
+        $self->_months;
 
         if ($self->{tokens}->[$self->{index}] =~ /^at$/i) {
             next;
-        } elsif ($self->{tokens}->[$self->{index}] =~ /^(\d{1,2})(:\d{2})?(am|pm)?|(noon|midnight)$/i) {
+        } elsif ($self->{tokens}->[$self->{index}] =~ /^(\d{1,2})(\:\d{2})?(am|pm)?|(noon|midnight)$/i) {
             my $hour_token    = $1;
             my $min_token     = $2;
             my $timeframe     = $3;
@@ -96,7 +88,7 @@ sub parse_datetime {
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^\d{4}$/) {
-            $self->{year} = $self->{tokens}->[$self->{index}];
+            $self->{datetime}->set_year($self->{tokens}->[$self->{index}]);
         }
 
         if ($self->{tokens}->[$self->{index}] !~ /^(?:this|next|last)$/i
@@ -130,8 +122,9 @@ sub parse_datetime {
          $self->_monthdays_limit;
          $self->_day;
 
-         return $self->_return_dt_object;
     }
+
+    return $self->_return_dt_object;
 }
 
 sub _init_data {
@@ -148,9 +141,20 @@ sub _init_data {
                                                October November December);
 }
 
+sub _set_modified { $_[0]->{modified} = 1 }
+
 sub _return_dt_object {
     my $self = shift;
 
+    die "$self->{date_string} not valid input, exiting.\n" unless $self->{modified};
+
+    $self->{year}  = $self->{datetime}->year;
+    $self->{month} = $self->{datetime}->month;
+    $self->{day}   = $self->{datetime}->day_of_month;
+    $self->{hour}  = $self->{datetime}->hour;
+    $self->{min}   = $self->{datetime}->minute;
+    $self->{sec}   = $self->{datetime}->second;
+
     $self->{sec}   = "0$self->{sec}"   unless length($self->{sec})   == 2;
     $self->{min}   = "0$self->{min}"   unless length($self->{min})   == 2;
     $self->{hour}  = "0$self->{hour}"  unless length($self->{hour})  == 2;
@@ -172,19 +176,20 @@ sub _ago {
     my @new_tokens = splice(@{$self->{tokens}}, $self->{index}, 3);
 
     if ($new_tokens[1] =~ /^hour(?:s)?$/i) {
-        $self->{hour} -= $new_tokens[0];
-    }
-    if ($new_tokens[1] =~ /^day(?:s)?$/i) {
-        $self->{day} -= $new_tokens[0];
-    }
-    if ($new_tokens[1] =~ /^week(?:s)?$/i) {
-        $self->{day} -= 7 * $new_tokens[0];
-    }
-    if ($new_tokens[1] =~ /^month(?:s)?$/i) {
-        $self->{month} -= $new_tokens[0];
-    }
-    if ($new_tokens[1] =~ /^year(?:s)?$/i) {
-        $self->{year} -= $new_tokens[0];
+        $self->{datetime}->subtract(hours => $new_tokens[0]);
+        $self->_set_modified();
+    } elsif ($new_tokens[1] =~ /^day(?:s)?$/i) {
+        $self->{datetime}->subtract(days => $new_tokens[0]);
+        $self->_set_modified();
+    } elsif ($new_tokens[1] =~ /^week(?:s)?$/i) {
+        $self->{datetime}->subtract(days => (7 * $new_tokens[0]));
+        $self->_set_modified();
+    } elsif ($new_tokens[1] =~ /^month(?:s)?$/i) {
+        $self->{datetime}->subtract(months => $new_tokens[0]);
+        $self->_set_modified();
+    } elsif ($new_tokens[1] =~ /^year(?:s)?$/i) {
+        $self->{datetime}->subtract(years => $new_tokens[0]);
+        $self->_set_modified();
     }
 }
 
@@ -195,30 +200,35 @@ sub _now {
 
     if ($new_tokens[1] =~ /^day(?:s)?$/i) {
         if ($new_tokens[2] =~ /^before$/i) {
-            $self->{day} -= $new_tokens[0];
+            $self->{datetime}->subtract(days => $new_tokens[0]);
+            $self->_set_modified();
         } elsif ($new_tokens[2] =~ /^from$/i) {
-            $self->{day} += $new_tokens[0];
+            $self->{datetime}->add(days => $new_tokens[0]);
+            $self->_set_modified();
         }
-    }
-    if ($new_tokens[1] =~ /^week(?:s)?$/i) {
+    } elsif ($new_tokens[1] =~ /^week(?:s)?$/i) {
         if ($new_tokens[2] =~ /^before$/i) {
-            $self->{day} -= 7 * $new_tokens[0];
+            $self->{datetime}->subtract(days => (7 * $new_tokens[0]));
+            $self->_set_modified();
         } elsif ($new_tokens[2] =~ /^from$/i) {
-            $self->{day} += 7 * $new_tokens[0];
+            $self->{datetime}->add(days => (7 * $new_tokens[0]));
+            $self->_set_modified();
         }
-     }
-     if ($new_tokens[1] =~ /^month(?:s)?$/i) {
+     } elsif ($new_tokens[1] =~ /^month(?:s)?$/i) {
          if ($new_tokens[2] =~ /^before$/i) {
-             $self->{month} -= $new_tokens[0];
+             $self->{datetime}->subtract(months => $new_tokens[0]);
+             $self->_set_modified();
          } elsif ($new_tokens[2] =~ /^from$/i) {
-             $self->{month} += $new_tokens[0];
+             $self->{datetime}->add(months => $new_tokens[0]);
+             $self->_set_modified();
          }
-     }
-     if ($new_tokens[1] =~ /^year(?:s)?$/i) {
+     } elsif ($new_tokens[1] =~ /^year(?:s)?$/i) {
          if ($new_tokens[2] =~ /^before$/i) {
-             $self->{year} -= $new_tokens[0];
+             $self->{datetime}->subtract(years => $new_tokens[0]);
+             $self->_set_modified();
          } elsif ($new_tokens[2] =~ /^from$/i) {
-             $self->{year} += $new_tokens[0];
+             $self->{datetime}->add(years => $new_tokens[0]);
+             $self->_set_modified();
          }
      }
 }
@@ -233,17 +243,20 @@ sub _daytime {
         $hour_token = $self->{tokens}->[$self->{index}-3];
     }
     if ($self->{tokens}->[$self->{index}] =~ /^morning$/i) {
-        $self->{hour} = $hour_token ? $hour_token : '08' - $self->{hours_before};
+        $self->{datetime}->set_hour($hour_token ? $hour_token : '08' - $self->{hours_before});
         undef $self->{hours_before};
+        $self->_set_modified();
     } elsif ($self->{tokens}->[$self->{index}] =~ /^afternoon$/i) {
-        $self->{hour} = $hour_token ? $hour_token + 12 : '14' - $self->{hours_before};
+        $self->{datetime}->set_hour($hour_token ? $hour_token + 12 : '14' - $self->{hours_before});
         undef $self->{hours_before};
+        $self->_set_modified();
     } else {
-        $self->{hour} = $hour_token ? $hour_token + 12 : '20' - $self->{hours_before};
+        $self->{datetime}->set_hour( $hour_token ? $hour_token + 12 : '14' - $self->{hours_before});
         undef $self->{hours_before};
+        $self->_set_modified();
     }
 
-    $self->{min} = '00';
+    $self->{datetime}->set_minute(00);
 }
 
 sub _months {
@@ -253,11 +266,12 @@ sub _months {
         my $key_month_short = substr($key_month, 0, 3);
         if ($self->{tokens}->[$self->{index}] =~ /$key_month/i
             || $self->{tokens}->[$self->{index}] =~ /$key_month_short/i) {
-            $self->{month} = $self->{months}->{$key_month};
+            $self->{datetime}->set_month($self->{months}->{$key_month});
+            $self->_set_modified();
             if ($self->{tokens}->[$self->{index}+1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
-                $self->{day} = $1;
+                $self->{datetime}->set_day($1);
             } elsif ($self->{tokens}->[$self->{index}-1] =~ /^(\d{1,2})(?:st|nd|rd|th)? ?$/i) {
-                  $self->{day} = $1;
+                  $self->{datetime}->set_day($1);
             }
             splice(@{$self->{tokens}}, $self->{index}, 2);
         }
@@ -270,16 +284,22 @@ sub _number {
     return if $self->{tokens}->[$self->{index}+1] eq 'in';
 
     if ($self->{tokens}->[$self->{index}+1] =~ /month(?:)/i) {
-        $self->{month} += $often;
-        $self->{month} -= 12 if $self->{month} > 12;
+        $self->{datetime}->add(months => $often);
+        if ($self->{datetime}->month() > 12) {
+            $self->{datetime}->subtract(months => 12);
+        }
+        $self->_set_modified();
     } elsif ($self->{tokens}->[$self->{index}+1] =~ /hour(?:s)/i) {
         if ($self->{tokens}->[$self->{index}+2] =~ /before/i) {
             $self->{hours_before} = $often;
+            $self->_set_modified();
         } elsif ($self->{tokens}->[$self->{index}+2] =~ /after/i) {
             $self->{hours_after} = $often;
+            $self->_set_modified();
         }
     } else {
-        $self->{day} = $often;
+        $self->{datetime}->set_day($often);
+        $self->_set_modified();
     }
 }
 
@@ -289,30 +309,36 @@ sub _at {
     if (!$timeframe && $self->{tokens}->[$self->{index}+1] 
         && $self->{tokens}[$self->{index}+1] =~ /^[ap]m$/i) {
         $timeframe = $self->{tokens}[$self->{index}+1];
-        $self->{index}++;
     }
 
     if ($hour_token) {
-        $self->{hour} = $hour_token;
+        $self->{datetime}->set_hour($hour_token);
         $min_token =~ s!:!! if defined($min_token);
-        $self->{min}  = $min_token || '00';
+        $self->{datetime}->set_minute($min_token || 00);
+
+        $self->_set_modified();
 
         if ($timeframe) {
             if ($timeframe =~ /^pm$/i) {
-                $hour_token += 12;
-                $self->{hour} = $hour_token;
-                $self->{min}  = 0 unless $min_token;
+                $self->{datetime}->add(hours => 12);
+                unless ($min_token) {
+                    $self->{datetime}->set_minute(0);
+                }
             }
         }
     } elsif ($noon_midnight) {
+        $self->_set_modified();
+        $self->{hours_before} ||= 0;
         if ($noon_midnight =~ /noon/i) {
-            ($self->{hour}, $self->{min}) = (12,0);
-            $self->{hour} -= $self->{hours_before};
+            $self->{datetime}->set_hour(12);
+            $self->{datetime}->set_minute(0);
+            $self->{datetime}->subtract(hours => $self->{hours_before});
             undef $self->{hours_before};
         } elsif ($noon_midnight =~ /midnight/i) {
-            ($self->{hour}, $self->{min}) = (0,0);
-            $self->{hour} -= $self->{hours_before};
-            $self->{day}++;
+            $self->{datetime}->set_hour(0);
+            $self->{datetime}->set_minute(0);
+            $self->{datetime}->subtract(hours => $self->{hours_before});
+            $self->{datetime}->add(days => 1);
         }
     }
 }
@@ -322,13 +348,17 @@ sub _weekday {
 
     foreach my $key_weekday (keys %{$self->{weekdays}}) {
         my $weekday_short = lc(substr($key_weekday,0,3));
-
         if ($self->{tokens}->[$self->{index}] =~ /$key_weekday/i || $self->{tokens}->[$self->{index}] eq $weekday_short) {
             $key_weekday = ucfirst(lc($key_weekday));
-            my $days_diff = $self->{weekdays}->{$key_weekday} > $self->{wday}
-              ? $self->{weekdays}->{$key_weekday} - $self->{wday}
-              : $self->{wday} - $self->{weekdays}->{$key_weekday};
-            $self->{day} += $days_diff;
+            my $days_diff;
+            if ($self->{weekdays}->{$key_weekday} > $self->{datetime}->wday) {
+                $days_diff = $self->{weekdays}->{$key_weekday} - $self->{datetime}->wday;
+                $self->{datetime}->add(days => $days_diff);
+            } else {
+                $days_diff = $self->{datetime}->wday - $self->{weekdays}->{$key_weekday};
+                $self->{datetime}->subtract(days => $days_diff);
+            }
+            $self->_set_modified();
             last;
         }
     }
@@ -341,15 +371,19 @@ sub _this_in {
         my $weekday_short = lc(substr($key_weekday,0,3));
 
         if ($self->{tokens}->[$self->{index}] =~ /$key_weekday/i || $self->{tokens}->[$self->{index}] eq $weekday_short) {
-            my $days_diff = $self->{weekdays}->{$key_weekday} - $self->{wday};
-            $self->{day} += $days_diff; $self->{buffer} = '';
+            my $days_diff = $self->{weekdays}->{$key_weekday} - $self->{datetime}->wday;
+            $self->{datetime}->add(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^week$/i) {
             my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}-2]));
-            my $days_diff = Decode_Day_of_Week($weekday) - $self->{wday};
-            $self->{day} += $days_diff; $self->{buffer} = '';
+            my $days_diff = Decode_Day_of_Week($weekday) - $self->{datetime}->wday;
+            $self->{datetime}->add(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
             last;
         }
 
@@ -360,8 +394,14 @@ sub _this_in {
                     if ($self->{tokens}->[$self->{index}-2] =~ /$weekday/i) {
 
                         my ($often) = $self->{tokens}->[$self->{index}-3] =~ /^(\d{1,2})(?:st|nd|rd|th)?$/i;
-                        ($self->{year}, $self->{month}, $self->{day}) =
-                        Nth_Weekday_of_Month_Year($self->{year}, $self->{months}->{$month}, $self->{weekdays}->{$weekday}, $often);
+                        my ($year, $month, $day) =
+                        Nth_Weekday_of_Month_Year($self->{datetime}->year, $self->{months}->{$month}, 
+                                                  $self->{weekdays}->{$weekday}, $often);
+                        $self->{datetime}->set_year($year);
+                        $self->{datetime}->set_month($month);
+                        $self->{datetime}->set_day($day);
+
+                        $self->_set_modified();
                     }
                 }
             }
@@ -376,25 +416,31 @@ sub _next {
         my $weekday_short = lc(substr($key_weekday,0,3));
 
         if ($self->{tokens}->[$self->{index}] =~ /$key_weekday/i || $self->{tokens}->[$self->{index}] eq $weekday_short) {
-            my $days_diff = (7 - $self->{wday}) + Decode_Day_of_Week($key_weekday);
-            $self->{day} += $days_diff; $self->{buffer} = '';
+            my $days_diff = (7 - $self->{datetime}->wday) + Decode_Day_of_Week($key_weekday);
+            $self->{datetime}->add(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^week$/i) {
             my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}-2]));
-            my $days_diff = (7 - $self->{wday}) + Decode_Day_of_Week($weekday);
-            $self->{day} += $days_diff; $self->{buffer} = '';
+            my $days_diff = (7 - $self->{datetime}->wday) + Decode_Day_of_Week($weekday);
+            $self->{datetime}->add(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^month$/i) {
-            $self->{month}++;
+            $self->{datetime}->add(months => 1);
+            $self->_set_modified();
             last;
         }
 
         if ($self->{tokens}->[$self->{index}] =~ /^year$/i) {
-            $self->{year}++;
+            $self->{datetime}->add(years => 1);
+            $self->_set_modified();
             last;
         }
     }
@@ -407,8 +453,10 @@ sub _last {
         my $weekday_short = lc(substr($key_weekday,0,3));
 
         if ($self->{tokens}->[$self->{index}] =~ /$key_weekday/i || $self->{tokens}->[$self->{index}] eq $weekday_short) {
-            my $days_diff = $self->{wday} + (7 - $self->{weekdays}->{$key_weekday});
-            $self->{day} -= $days_diff; $self->{buffer} = '';
+            my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$key_weekday});
+            $self->{datetime}->subtract(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
             last;
         }
     }
@@ -417,34 +465,44 @@ sub _last {
 
         if (exists $self->{weekdays}->{ucfirst(lc($self->{tokens}->[$self->{index}+1]))}) {
             my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}+1]));
-            my $days_diff = $self->{wday} + (7 - $self->{weekdays}->{$weekday});
-            $self->{day} -= $days_diff; $self->{buffer} = '';
+            my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$weekday});
+            $self->{datetime}->subtract(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
         } elsif (exists $self->{weekdays}->{ucfirst(lc($self->{tokens}->[$self->{index}-2]))}) {
             my $weekday = ucfirst(lc($self->{tokens}->[$self->{index}-2]));
-            my $days_diff = $self->{wday} + (7 - $self->{weekdays}->{$weekday});
-            $self->{day} -= $days_diff; $self->{buffer} = '';
+            my $days_diff = $self->{datetime}->wday + (7 - $self->{weekdays}->{$weekday});
+            $self->{datetime}->subtract(days => $days_diff);
+            $self->{buffer} = '';
+            $self->_set_modified();
         }
     }
 
     if ($self->{tokens}->[$self->{index}] =~ /^month$/i) {
-        $self->{month}--;
+        $self->{datetime}->subtract(months => 1);
+        $self->_set_modified();
     }
 
     if ($self->{tokens}->[$self->{index}] =~ /^year$/i) {
-        $self->{year}--;
+        $self->{datetime}->subtract(years => 1);
+        $self->_set_modified();
     }
 }
 
 sub _monthdays_limit {
     my $self = shift;
 
-    my $monthdays = Days_in_Month($self->{year}, $self->{month});
+    my $monthdays = Days_in_Month($self->{datetime}->year, $self->{datetime}->month);
 
-    if ($self->{day} > $monthdays) {
-        $self->{month}++; $self->{day} = $self->{day} - $monthdays;
-    } elsif ($self->{day} < 1) {
-        $monthdays = Days_in_Month($self->{year}, ($self->{month}-1));
-        $self->{month}--; $self->{day} = $monthdays - $self->{day};
+    if ($self->{datetime}->day > $monthdays) {
+        $self->{datetime}->add(months => 1);
+        $self->{datetime}->set_day($self->{datetime}->day - $monthdays);
+        $self->_set_modified();
+    } elsif ($self->{datetime}->day < 1) {
+        $monthdays = Days_in_Month($self->{datetime}->year, ($self->{datetime}->month-1));
+        $self->{datetime}->subtract(months => 1);
+        $self->{datetime}->set_day($monthdays - $self->{datetime}->day);
+        $self->_set_modified();
     }
 }
 
@@ -452,20 +510,26 @@ sub _day {
     my $self = shift;
 
     if ($self->{tokens}->[$self->{index}] =~ /^(?:today|yesterday|tomorrow)$/i) {
-        $self->{day}-- if $self->{tokens}->[$self->{index}] =~ /yesterday/i;
-        $self->{day}++ if $self->{tokens}->[$self->{index}] =~ /tomorrow/i;
+        if ($self->{tokens}->[$self->{index}] =~ /yesterday/i) {
+            $self->{datetime}->subtract(days => 1);
+        }
+        if ($self->{tokens}->[$self->{index}] =~ /tomorrow/i) {
+            $self->{datetime}->add(days => 1);
+        }
+
+        $self->_set_modified();
 
         if ($self->{hours_before}) {
-            $self->{hour} = 24 - $self->{hours_before};
-            $self->{day}--;
+            $self->{datetime}->set_hour(24 - $self->{hours_before});
+            $self->{datetime}->subtract(days => 1);
         } elsif ($self->{hours_after}) {
-            $self->{hour} = 0 + $self->{hours_after};
+            $self->{datetime}->set_hour(0 + $self->{hours_after});
         }
     }
 
-    if ($self->{hour} < 0) {
-        my ($subtract) = $self->{hour} =~ /\-(.*)/;
-        $self->{hour} = 12 - $subtract;
+    if ($self->{datetime}->hour < 0) {
+        my ($subtract) = $self->{datetime}->hour =~ /\-(.*)/;
+        $self->{datetime}->set_hour(12 - $subtract);
     }
 }
 
@@ -555,6 +619,7 @@ Below are some examples of human readable date/time input:
  3rd wednesday in november
  3rd month next year
  7 hours before tomorrow at noon
+ 3 months ago saturday at 5:00pm
 
 =head2 Specific Dates
 
